// SPDX-License-Identifier: MIT

use anyhow::Context;
use axum::{
    response::IntoResponse,
    routing::{delete, post},
    Router,
};
use regex::bytes::Regex;
use serde::Deserialize;
use serde_json::json;

use crate::common::{
    self, json_response, CheckAuthRestApi, JsonOrForm, RestApiPermissionNetworkAdmin,
};
use crate::error::ErrorStringResult;

pub fn routes() -> Router {
    Router::new()
        .route("/api/wwan", post(rest_wwan_setup))
        .route("/api/wwan", delete(rest_wwan_delete))
}

// connect output is something like this on success (including reconnects,
// which re-use the same id)
//   Setup WWAN connection 'xxxx-xxxx...'
const CONNECT_RE: &str = r#"Setup WWAN connection '([^']*)'"#;

#[derive(Deserialize)]
#[serde(deny_unknown_fields)]
struct WwanSetupParam {
    apn: String,
    user: Option<String>,
    password: Option<String>,
    auth_type: Option<String>,
    mccmnc: Option<String>,
    device: Option<String>,
    ipv6: Option<bool>,
}

/// POST "/api/wwan"
/// - Access: Network admin
/// - Input: APN, user, password, auth_type, mccmnc,
///   device, ipv6 (bool)
/// - Output: None
async fn rest_wwan_setup(
    _auth: CheckAuthRestApi<RestApiPermissionNetworkAdmin>,
    JsonOrForm(params): JsonOrForm<WwanSetupParam>,
) -> ErrorStringResult {
    let mut args = vec![
        "wwan_setup.sh",
        common::WWAN_CON_NAME,
        params.device.as_deref().unwrap_or("ttyCommModem"),
        &params.apn,
        params.auth_type.as_deref().unwrap_or("CHAP"),
    ];
    if params.ipv6.unwrap_or(true) {
        args.push("auto")
    } else {
        args.push("ignore")
    }
    if let Some(user) = &params.user {
        args.push("--user");
        args.push(user);
    }
    if let Some(password) = &params.password {
        args.push("--password");
        args.push(password);
    }
    if let Some(mccmnc) = &params.mccmnc {
        args.push("--mccmnc");
        args.push(mccmnc)
    }
    // build RE before connecting for error handling
    let re = Regex::new(CONNECT_RE).context("Could not build regex?!")?;

    let output = common::exec_command(&args).await?;
    let uuid = re
        .captures(&output.stdout)
        .and_then(|c| c.get(1))
        .context("Connected, but could not get uuid")?;

    json_response(&json!({
        "uuid": String::from_utf8_lossy(uuid.as_bytes()),
    }))
}

/// DELETE "/api/wwan"
/// - Access: Network admin
/// - Input: None
/// - Output: None
async fn rest_wwan_delete(
    _auth: CheckAuthRestApi<RestApiPermissionNetworkAdmin>,
) -> ErrorStringResult {
    common::exec_command(&["wwan_delete.sh", common::WWAN_CON_NAME]).await?;
    Ok(().into_response())
}
